@@ -99,21 +99,18 @@
   :link '(custom-manual "(haskell-mode)Indentation")
   :prefix "haskell-indent-")
 
-;;;###autoload
 (defcustom haskell-indent-offset 4
   "Indentation of Haskell statements with respect to containing block."
   :type 'integer
   :safe #'natnump
   :group 'haskell-indent)
 
-;;;###autoload
 (defcustom haskell-indent-literate-Bird-default-offset 1
   "Default number of blanks after > in a Bird style literate script."
   :type 'integer
   :safe #'natnump
   :group 'haskell-indent)
 
-;;;###autoload
 (defcustom haskell-indent-rhs-align-column 0
   "Column on which to align right-hand sides (use 0 for ad-hoc alignment)."
   :type 'integer
@@ -326,7 +323,6 @@ It deals with both Bird style and non Bird-style scripts."
         (insert "\\begin{code}\n")))))
 
 ;;; Start of indentation code
-;;;###autoload
 (defcustom haskell-indent-look-past-empty-line t
   "If nil, indentation engine will not look past an empty line for layout points."
   :group 'haskell-indent
@@ -1022,7 +1018,6 @@ OPEN is the start position of the comment in which point is."
                             (haskell-indent-point-to-col (match-end 0)))
                         (haskell-indent-point-to-col (point))))))))))
 
-;;;###autoload
 (defcustom haskell-indent-thenelse 0
   "If non-nil, \"then\" and \"else\" are indented.
 This is necessary in the \"do\" layout under Haskell-98.
@@ -1054,7 +1049,6 @@ See http://hackage.haskell.org/trac/haskell-prime/wiki/DoAndIfThenElse"
     (list (list (+ (if (memq (char-after) '(?t ?e)) haskell-indent-thenelse 0)
                    (haskell-indent-point-to-col open))))))
 
-;;;###autoload
 (defcustom haskell-indent-after-keywords
   '(("where" 2 0)
     ("of" 2)
@@ -1103,7 +1097,6 @@ is at the end of an otherwise-non-empty line."
     (or (assoc id haskell-indent-after-keywords)
         (car (member id haskell-indent-after-keywords)))))
 
-;;;###autoload
 (defcustom haskell-indent-dont-hang '("(")
   "Lexemes that should never be considered as hanging."
   :group 'haskell-indent
@@ -1506,15 +1499,15 @@ One indentation cycle is used."
     ;; (define-key map "\177"  'backward-delete-char-untabify)
     ;; The binding to TAB is already handled by indent-line-function.  --Stef
     ;; (define-key map "\t"    'haskell-indent-cycle)
-    (define-key map [?\C-c ?\C-=] 'haskell-indent-insert-equal)
-    (define-key map [?\C-c ?\C-|] 'haskell-indent-insert-guard)
+    (define-key map (kbd "C-c C-=") 'haskell-indent-insert-equal)
+    (define-key map (kbd "C-c C-|") 'haskell-indent-insert-guard)
     ;; Alternate binding, in case C-c C-| is too inconvenient to type.
     ;; Duh, C-g is a special key, let's not use it here.
-    ;; (define-key map [?\C-c ?\C-g] 'haskell-indent-insert-guard)
-    (define-key map [?\C-c ?\C-o] 'haskell-indent-insert-otherwise)
-    (define-key map [?\C-c ?\C-w] 'haskell-indent-insert-where)
-    (define-key map [?\C-c ?\C-.] 'haskell-indent-align-guards-and-rhs)
-    (define-key map [?\C-c ?\C->] 'haskell-indent-put-region-in-literate)
+    ;; (define-key map (kbd "C-c C-g") 'haskell-indent-insert-guard)
+    (define-key map (kbd "C-c C-o") 'haskell-indent-insert-otherwise)
+    (define-key map (kbd "C-c C-w") 'haskell-indent-insert-where)
+    (define-key map (kbd "C-c C-.") 'haskell-indent-align-guards-and-rhs)
+    (define-key map (kbd "C-c C->") 'haskell-indent-put-region-in-literate)
     map))
 
 ;;;###autoload
@@ -1531,9 +1524,8 @@ One indentation cycle is used."
   (let ((map (current-local-map)))
     (while (and map (not (eq map haskell-indent-map)))
       (setq map (keymap-parent map)))
-    (if map
-        ;; haskell-indent-map is already active: nothing to do.
-        nil
+    ;; if haskell-indent-map is already active: there's nothing to do.
+    (unless map
       ;; Put our keymap on top of the others.  We could also put it in
       ;; second place, or in a minor-mode.  The minor-mode approach would be
       ;; easier, but it's harder for the user to override it.  This approach
