@@ -52,7 +52,6 @@
 
 ;; Here I depart from the inferior-haskell- prefix.
 ;; Not sure if it's a good idea.
-;;;###autoload
 (defcustom haskell-program-name
   ;; Arbitrarily give preference to hugs over ghci.
   (or (cond
@@ -111,7 +110,6 @@ The command can include arguments."
   "Regexps for error messages generated by inferior Haskell processes.
 The format should be the same as for `compilation-error-regexp-alist'.")
 
-;;;###autoload
 (defcustom inferior-haskell-find-project-root t
   "If non-nil, try and find the project root directory of this file.
 This will either look for a Cabal file or a \"module\" statement in the file."
@@ -207,7 +205,6 @@ setting up the inferior-haskell buffer."
   (let ((proc (inferior-haskell-process arg)))
     (pop-to-buffer (process-buffer proc))))
 
-;;;###autoload
 (defcustom inferior-haskell-wait-and-jump nil
   "If non-nil, wait for file loading to terminate and jump to the error."
   :type 'boolean
@@ -218,9 +215,9 @@ setting up the inferior-haskell buffer."
 
 (defun inferior-haskell-send-decl-post-filter (string)
   (when (and inferior-haskell-send-decl-post-filter-on
-             #1=(string-match inferior-haskell-multiline-prompt-re string))
+             (string-match inferior-haskell-multiline-prompt-re string))
     ;; deleting sequence of `%s|' multiline promts
-    (while #1#
+    (while (string-match inferior-haskell-multiline-prompt-re string)
       (setq string (substring string (match-end 0))))
     ;; deleting regular prompts
     (setq string (replace-regexp-in-string comint-prompt-regexp "" string)
@@ -275,7 +272,7 @@ The process PROC should be associated to a comint buffer."
            )
       (or (when cabal
             (with-current-buffer cabal
-              (let ((hsd (haskell-cabal-get-setting "hs-source-dirs")))
+              (let ((hsd (haskell-cabal--get-field "hs-source-dirs")))
                 (if (null hsd)
                     ;; If there's a Cabal file with no Hs-Source-Dirs, then
                     ;; just use the Cabal file's directory.
@@ -588,7 +585,6 @@ The returned info is cached for reuse by `haskell-doc-mode'."
 ;; * Support fetching of local Haddock docs pulled directly from source files.
 ;; * Display docs locally? w3m?
 
-;;;###autoload
 (defcustom inferior-haskell-use-web-docs
   'fallback
   "Whether to use the online documentation.  Possible values:
@@ -602,7 +598,6 @@ regardless of existance of local files.  Default is `fallback'."
                  (const :tag "As fallback" fallback)
                  (const :tag "Always" always)))
 
-;;;###autoload
 (defcustom inferior-haskell-web-docs-base
   "http://haskell.org/ghc/docs/latest/html/libraries/"
   "The base URL of the online libraries documentation.
@@ -611,13 +606,11 @@ is `always' or `fallback'."
   :group 'inferior-haskell
   :type 'string)
 
-;;;###autoload
 (defcustom haskell-package-manager-name "ghc-pkg"
   "Name of the program to consult regarding package details."
   :group 'inferior-haskell
   :type 'string)
 
-;;;###autoload
 (defcustom haskell-package-conf-file
   (condition-case nil
       (with-temp-buffer
@@ -687,7 +680,6 @@ Insert the output into the current buffer."
       (message "Generating module alist... done")
       module-alist)))
 
-;;;###autoload
 (defcustom inferior-haskell-module-alist-file
   ;; (expand-file-name "~/.inf-haskell-module-alist")
   (expand-file-name (concat "inf-haskell-module-alist-"
@@ -805,26 +797,26 @@ we load it."
 
 (defvar inf-haskell-mode-map
   (let ((map (make-sparse-keymap)))
-    ;; (define-key map [?\M-C-x]     'inferior-haskell-send-defun)
-    ;; (define-key map [?\C-x ?\C-e] 'inferior-haskell-send-last-sexp)
-    ;; (define-key map [?\C-c ?\C-r] 'inferior-haskell-send-region)
-    (define-key map [?\C-x ?\C-d] 'inferior-haskell-send-decl)
-    (define-key map [?\C-c ?\C-z] 'switch-to-haskell)
-    (define-key map [?\C-c ?\C-l] 'inferior-haskell-load-file)
+    ;; (define-key map (kbd "M-C-x")    'inferior-haskell-send-defun)
+    ;; (define-key map (kbd "C-x C-e") 'inferior-haskell-send-last-sexp)
+    ;; (define-key map (kbd "C-c C-r") 'inferior-haskell-send-region)
+    (define-key map (kbd "C-x C-d") 'inferior-haskell-send-decl)
+    (define-key map (kbd "C-c C-z") 'switch-to-haskell)
+    (define-key map (kbd "C-c C-l") 'inferior-haskell-load-file)
     ;; I think it makes sense to bind inferior-haskell-load-and-run to C-c
     ;; C-r, but since it used to be bound to `reload' until June 2007, I'm
     ;; going to leave it out for now.
-    ;; (define-key map [?\C-c ?\C-r] 'inferior-haskell-load-and-run)
-    (define-key map [?\C-c ?\C-b] 'switch-to-haskell)
-    ;; (define-key map [?\C-c ?\C-s] 'inferior-haskell-start-process)
+    ;; (define-key map (kbd "C-c C-r") 'inferior-haskell-load-and-run)
+    (define-key map (kbd "C-c C-b") 'switch-to-haskell)
+    ;; (define-key map (kbd "C-c C-s") 'inferior-haskell-start-process)
     ;; That's what M-; is for.
     (define-key map (kbd "C-c C-t") 'inferior-haskell-type)
     (define-key map (kbd "C-c C-i") 'inferior-haskell-info)
     (define-key map (kbd "C-c M-.") 'inferior-haskell-find-definition)
     (define-key map (kbd "C-c C-d") 'inferior-haskell-find-haddock)
-    (define-key map [?\C-c ?\C-v] 'haskell-check)
+    (define-key map (kbd "C-c C-v") 'haskell-check)
     map)
-  "Keymap for using inf-haskell.")
+  "Keymap for using `inf-haskell-mode'.")
 
 ;;;###autoload
 (define-minor-mode inf-haskell-mode
